@@ -1,6 +1,6 @@
 // BC_database_reader.cpp
 //
-// Copyright 2012-2013 Roan Trail, Inc.
+// Copyright 2012-2014 Roan Trail, Inc.
 //
 // This file is part of Tovero.
 //
@@ -61,6 +61,8 @@
 #include <tovero/math/geometry/Geometric_tolerances.hpp>
 #include <tovero/math/geometry/Half_space.hpp>
 #include <tovero/math/geometry/Matrix.hpp>
+#include <tovero/math/geometry/Pipe.hpp>
+#include <tovero/math/geometry/Pipe_control_point.hpp>
 #include <tovero/math/geometry/Plate_triangle_mesh.hpp>
 #include <tovero/math/geometry/Point.hpp>
 #include <tovero/math/geometry/Polyhedron.hpp>
@@ -101,6 +103,8 @@ using Roan_trail::Tovero_math::General_cone;
 using Roan_trail::Tovero_math::Geometric_tolerances;
 using Roan_trail::Tovero_math::Half_space;
 using Roan_trail::Tovero_math::Matrix;
+using Roan_trail::Tovero_math::Pipe;
+using Roan_trail::Tovero_math::Pipe_control_point;
 using Roan_trail::Tovero_math::Plate_triangle_mesh;
 using Roan_trail::Tovero_math::Point;
 using Roan_trail::Tovero_math::Polyhedron;
@@ -146,12 +150,30 @@ namespace
     return op;
   }
 
-  void ih_copy_matrix(const matp_t BC_matrix, Matrix& return_matrix)
+  void ih_copy_matrix(const matp_t BC_matrix,
+                      Matrix& return_matrix,
+                      const Distance& conversion_factor)
   {
     for (size_t i = 0; i < 16; ++i)
     {
       return_matrix[i] = BC_matrix[i];
     }
+
+    double tx;
+    double ty;
+    double tz;
+
+    return_matrix.translation_values(tx,
+                                     ty,
+                                     tz);
+
+    tx *= conversion_factor.value();
+    ty *= conversion_factor.value();
+    tz *= conversion_factor.value();
+
+    return_matrix.set_translation_values(tx,
+                                         ty,
+                                         tz);
   }
 }
 
@@ -661,8 +683,10 @@ void BC_database_reader::mf_do_combination(struct db_i *database,
         if (edge_matrix)
         {
           Matrix matrix(Matrix::I);
-          ih_copy_matrix(edge_matrix, matrix);
-          transformation = new Transformation(matrix, *reader->m_distance_conversion_factor);
+          ih_copy_matrix(edge_matrix,
+                         matrix,
+                         *reader->m_distance_conversion_factor);
+          transformation = new Transformation(matrix);
         }
         // setup the member
         //   get operator for this member
@@ -770,6 +794,9 @@ void BC_database_reader::mf_do_leaf(struct db_i *database,
     case ID_HALF: // half-space defined by a plane and a distance from origin
       valid_solid = reader->mf_make_half_space(conversion, error);
       break;
+    case ID_PIPE:
+      valid_solid = reader->mf_make_pipe(conversion, error);
+      break;
     case ID_POLY: // polygons (up to 5 vertices per)
     case ID_BSPLINE: // NURB surfaces
     case ID_NMG: // N-manifold geometry
@@ -778,7 +805,6 @@ void BC_database_reader::mf_do_leaf(struct db_i *database,
     case ID_HF: // height field (terrain primitive), references an external file
     case ID_EBM: // extruded bit-map, references an external file
     case ID_VOL: // volume of revolution, references an external file
-    case ID_PIPE:
     case ID_PARTICLE:
     case ID_RPC:
     case ID_RHC:
@@ -915,13 +941,13 @@ bool BC_database_reader::mf_make_BOT(const Conversion_struct& conversion, Error_
                                   BC_BOT->faces[index + 2]));
   }
   // convert normals
-  vector<Vector> normals;
+  vector<Unit_vector> normals;
   for (size_t i = 0; i < BC_BOT->num_normals; ++i)
   {
     const size_t index = i * 3;
-    normals.push_back(Vector(BC_BOT->normals[index] * *m_distance_conversion_factor,
-                             BC_BOT->normals[index + 1] * *m_distance_conversion_factor,
-                             BC_BOT->normals[index + 2] * *m_distance_conversion_factor));
+    normals.push_back(Unit_vector(Unitless(BC_BOT->normals[index]),
+                                  Unitless(BC_BOT->normals[index + 1]),
+                                  Unitless(BC_BOT->normals[index + 2])));
   }
   vector<Triangle_face> face_normals;
   for (size_t i = 0; i < BC_BOT->num_face_normals; ++i)
@@ -1401,6 +1427,81 @@ bool BC_database_reader::mf_make_general_cone(const Conversion_struct& conversio
   return return_value;
 }
 
+// Documentation for BRL-CAD pipe structure:
+//
+// struct rt_pipe_internal {
+//   uint32_t pipe_magic;
+//   struct bu_list pipe_segs_head;
+//   // REMAINING ELEMENTS PROVIDED BY IMPORT, UNUSED BY EXPORT
+//   int pipe_count;
+// };
+
+bool BC_database_reader::mf_make_pipe(const Conversion_struct& conversion,
+                                      Error_param& return_error)
+{
+  precondition(conversion.solid
+               && (conversion.solid_ID == ID_PIPE)
+               && ("" != conversion.solid_name)
+               && !return_error()
+               && !m_node_mapper->find_node_by_name(conversion.solid_name));
+
+  bool return_value = false;
+
+  start_error_block();
+
+  const struct rt_pipe_internal* BC_pipe = static_cast<const rt_pipe_internal *>(conversion.solid);
+  RT_PIPE_CK_MAGIC(BC_pipe);
+
+  vector<Pipe_control_point> control_points;
+
+  const struct bu_list* BC_list_head = &BC_pipe->pipe_segs_head;
+  for (const struct bu_list* BC_current_element = BC_list_head;
+       BC_current_element && (BC_current_element != BC_list_head);
+       BC_current_element = BC_current_element->forw)
+  {
+    // Note: reinterpret cast is OK because the list is made of wdb_pipept structs
+    const wdb_pipept* BC_control_point = reinterpret_cast<const struct wdb_pipept*>(BC_current_element);
+    Pipe_control_point control_point(Point(BC_control_point->pp_coord[X] * *m_distance_conversion_factor,
+                                           BC_control_point->pp_coord[Y] * *m_distance_conversion_factor,
+                                           BC_control_point->pp_coord[Z] * *m_distance_conversion_factor),
+                                     BC_control_point->pp_id / 2.0  // diameter to radius
+                                     * *m_distance_conversion_factor,
+                                     BC_control_point->pp_od / 2.0  // diameter to radius
+                                     * *m_distance_conversion_factor,
+                                     BC_control_point->pp_bendradius * *m_distance_conversion_factor);
+    control_points.push_back(control_point);
+  }
+
+  Pipe* pipe = new Pipe(control_points);
+
+  // validate
+  Error_param error;
+  const bool valid = pipe->is_valid(m_tolerances, error);
+  if (!valid)
+  {
+    on_error(!m_tolerate_database_errors, new Graphics_error(error_location(),
+                                                             Graphics_error::validation,
+                                                             error()));
+    string diagnostic = error()->error_dictionary()[Error::diagnostic_error_key];
+    m_logger << Logger::warning << diagnostic << endl;
+  }
+
+  Solid* solid_to_add = pipe;
+  mf_add_solid(conversion.solid_name, solid_to_add);
+
+  return_value = true;
+  goto exit_point;
+
+  end_error_block();
+
+  default_error_handler_and_cleanup(return_error,
+                                    return_value,
+                                    false);
+ exit_point:
+  postcondition(return_error.is_valid_at_return(return_value));
+  return return_value;
+}
+
 void BC_database_reader::mf_add_solid(const string& solid_name, Solid* solid)
 {
   solid->reference();
