@@ -1,6 +1,6 @@
 // BC_database_writer.cpp
 //
-// Copyright 2012-2013 Roan Trail, Inc.
+// Copyright 2012-2014 Roan Trail, Inc.
 //
 // This file is part of Tovero.
 //
@@ -46,6 +46,8 @@
 #include <tovero/math/geometry/Half_space.hpp>
 #include <tovero/math/geometry/Oblique_cone.hpp>
 #include <tovero/math/geometry/Oblique_cylinder.hpp>
+#include <tovero/math/geometry/Pipe.hpp>
+#include <tovero/math/geometry/Pipe_control_point.hpp>
 #include <tovero/math/geometry/Plate_triangle_mesh.hpp>
 #include <tovero/math/geometry/Polyhedron.hpp>
 #include <tovero/math/geometry/Polyhedron_base.hpp>
@@ -102,6 +104,8 @@ using Roan_trail::Tovero_math::Geometric_tolerances;
 using Roan_trail::Tovero_math::Matrix;
 using Roan_trail::Tovero_math::Oblique_cone;
 using Roan_trail::Tovero_math::Oblique_cylinder;
+using Roan_trail::Tovero_math::Pipe;
+using Roan_trail::Tovero_math::Pipe_control_point;
 using Roan_trail::Tovero_math::Plate_triangle_mesh;
 using Roan_trail::Tovero_math::Point;
 using Roan_trail::Tovero_math::Polyhedron;
@@ -178,14 +182,23 @@ namespace
                          const Distance& conversion_factor,
                          mat_t& return_BC_matrix)
   {
-    Matrix m = transform.matrix();
-    const Distance& translation_units = transform.translation_units();
+    Matrix m = transform.matrix(); // make a copy to modify
+
+    double tx;
+    double ty;
+    double tz;
+    m.translation_values(tx,
+                         ty,
+                         tz);
 
     // convert units for translation only
-    const double translate_factor = (translation_units / conversion_factor).value();
-    m[3] *= translate_factor;
-    m[7] *= translate_factor;
-    m[11] *= translate_factor;
+    tx /= conversion_factor.value();
+    ty /= conversion_factor.value();
+    tz /= conversion_factor.value();
+
+    m.set_translation_values(tx,
+                             ty,
+                             tz);
 
     for (size_t i = 0; i < 16; ++i)
     {
@@ -562,8 +575,11 @@ Solid_visitor::Visit_result BC_database_writer::visit_exit(const Solid_operation
       is_region = BC_attributes.is_region();
       shader_name = BC_attributes.shader_name();
       shader_args = BC_attributes.shader_args();
-      BC_attributes.copy_color(BC_color_values);
-      BC_color = BC_color_values;
+      if (BC_attributes.has_color())
+      {
+        BC_attributes.copy_color(BC_color_values);
+        BC_color = BC_color_values;
+      }
       material = BC_attributes.material();
       children_inherit = BC_attributes.children_inherit();
     }
@@ -713,8 +729,11 @@ Solid_visitor::Visit_result BC_database_writer::visit_exit(const Solid_combinati
       is_region = BC_attributes.is_region();
       shader_name = BC_attributes.shader_name();
       shader_args = BC_attributes.shader_args();
-      BC_attributes.copy_color(BC_color_values);
-      BC_color = BC_color_values;
+      if (BC_attributes.has_color())
+      {
+        BC_attributes.copy_color(BC_color_values);
+        BC_color = BC_color_values;
+      }
       material = BC_attributes.material();
       children_inherit = BC_attributes.children_inherit();
     }
@@ -879,6 +898,82 @@ Solid_visitor::Visit_result BC_database_writer::visit(const Oblique_cylinder& cy
   return mf_visit_base(cylinder);
 }
 
+Solid_visitor::Visit_result BC_database_writer::visit(const Pipe& pipe)
+{
+  Solid_visitor::Visit_result return_value = Solid_visitor::abort;
+
+  struct bu_list pipe_list;
+  mk_pipe_init(&pipe_list);
+
+  start_error_block();
+
+  string solid_name;
+  Error_param error;
+  const bool success = mf_add_and_validate_solid(pipe,
+                                                 solid_name,
+                                                 error);
+  on_error(!success, new Graphics_error(error_location(),
+                                        Graphics_error::general,
+                                        error()));
+
+  const vector<Pipe_control_point>& control_points = pipe.control_points();
+  const size_t BC_control_point_count = control_points.size();
+  for (size_t i = 0; i < BC_control_point_count; ++i)
+  {
+    const Pipe_control_point& control_point = control_points[i];
+    const Point& position = control_point.position();
+    point_t point =
+    {
+      (position[0] / *m_distance_conversion_factor).value(),
+      (position[1] / *m_distance_conversion_factor).value(),
+      (position[2] / *m_distance_conversion_factor).value()
+    };
+    const double BC_outer_diameter = (control_point.outer_radius()
+                                      / *m_distance_conversion_factor).value() * 2.0; // radius to diameter
+    const double BC_inner_diameter = (control_point.inner_radius()
+                                      / *m_distance_conversion_factor).value() * 2.0; // radius to diameter
+    const double BC_bend_radius = (control_point.bend_radius() / *m_distance_conversion_factor).value();
+    mk_add_pipe_pt(&pipe_list,
+                   point,
+                   BC_outer_diameter,
+                   BC_inner_diameter,
+                   BC_bend_radius);
+  }
+
+  // make the BC solid
+  const int return_code = mk_pipe(m_file.database,
+                                  solid_name.c_str(),
+                                  &pipe_list);
+  if (return_code)
+  {
+    const string diagnostic = string("Could not create ") + pipe.solid_class() + string(": ") + solid_name;
+    on_error(!m_tolerate_database_errors, new Graphics_error(error_location(),
+                                                             Graphics_error::write,
+                                                             diagnostic));
+    m_logger << Logger::warning << diagnostic << endl;
+  }
+
+  return_value = Solid_visitor::success;
+  goto exit_point;
+
+  end_error_block();
+
+ error_handler:
+  delete m_IO_error;
+  m_IO_error = handler_error;
+  goto error_cleanup;
+
+ error_cleanup:
+  return_value = Solid_visitor::abort;
+  goto exit_point;
+
+ exit_point:
+  // cleanup
+  mk_pipe_free(&pipe_list);
+
+  return return_value;
+}
+
 Solid_visitor::Visit_result BC_database_writer::visit(const Plate_triangle_mesh& mesh)
 {
   return mf_visit_base(mesh,
@@ -1402,7 +1497,7 @@ Solid_visitor::Visit_result BC_database_writer::mf_visit_base(const Triangle_mes
     }
   }
   // normals
-  const vector<Vector>& normals = mesh.normals();
+  const vector<Unit_vector>& normals = mesh.normals();
   const size_t BC_normal_count = normals.size();
   unsigned char BC_bot_flags = 0;
   // flags
@@ -1415,16 +1510,16 @@ Solid_visitor::Visit_result BC_database_writer::mf_visit_base(const Triangle_mes
     BC_bot_flags |= RT_BOT_USE_NORMALS;
   }
   size_t BC_face_normal_count = 0;
-  if (mesh.has_surface_normals() && BC_normal_count)
+  if (BC_normal_count)
   {
     BC_normals = new fastf_t[BC_normal_count * 3];
     for (size_t i = 0; i < BC_normal_count; ++i)
     {
-      const Vector& normal = normals[i];
+      const Unit_vector& normal = normals[i];
       const size_t index = i * 3;
-      BC_normals[index] = (normal[0] / *m_distance_conversion_factor).value();
-      BC_normals[index + 1] = (normal[1] / *m_distance_conversion_factor).value();
-      BC_normals[index + 2] = (normal[0] / *m_distance_conversion_factor).value();
+      BC_normals[index] = normal[0].value();
+      BC_normals[index + 1] = normal[1].value();
+      BC_normals[index + 2] = normal[2].value();
     }
     const vector<Triangle_face>& face_normals = mesh.face_normals();
     BC_face_normal_count = face_normals.size();
