@@ -1,6 +1,6 @@
 // BC_shader.cpp
 //
-// Copyright 2012-2013 Roan Trail, Inc.
+// Copyright 2012-2014 Roan Trail, Inc.
 //
 // This file is part of Tovero.
 //
@@ -177,11 +177,11 @@ namespace
     {"%f", 1, "fract", bu_offsetof(struct light_specific, lt_fraction), 0, 0, 0 }, // check_code_ignore
     {"%f", 1, "f", bu_offsetof(struct light_specific, lt_fraction), 0, 0, 0 }, // check_code_ignore
     //
-    {"%f", 3, "target", bu_offsetofarray(light_specific, lt_target), 0, 0, 0 }, // check_code_ignore
-    {"%f", 3, "t", bu_offsetofarray(light_specific, lt_target), 0, 0, 0 }, // check_code_ignore
+    {"%f", 3, "target", bu_offsetof(light_specific, lt_target), 0, 0, 0 }, // check_code_ignore
+    {"%f", 3, "t", bu_offsetof(light_specific, lt_target), 0, 0, 0 }, // check_code_ignore
     // TODO: this is not in sh_light.c, but mged only accepts it (and "d")
-    {"%f", 3, "direct", bu_offsetofarray(light_specific, lt_target), 0, 0, 0 }, // check_code_ignore
-    {"%f", 3, "aim", bu_offsetofarray(light_specific, lt_target), 0, 0, 0 }, // check_code_ignore
+    {"%f", 3, "direct", bu_offsetof(light_specific, lt_target), 0, 0, 0 }, // check_code_ignore
+    {"%f", 3, "aim", bu_offsetof(light_specific, lt_target), 0, 0, 0 }, // check_code_ignore
 
     //
     {"%d", 1, "shadows", bu_offsetof(struct light_specific, lt_shadows), 0, 0, 0 }, // check_code_ignore
@@ -266,7 +266,7 @@ BC_shader::BC_shader(const string& shader)
     m_shader_args(),
     m_shader(0)
 {
-  // setup shader
+  // setup shader map
   if (!iv_shader_map.size())
   {
     ih_setup_shader_map();
