@@ -2,7 +2,7 @@
 #
 # bus.rb
 #
-# Copyright 2013 Roan Trail, Inc.
+# Copyright 2013-2014 Roan Trail, Inc.
 #
 # This file is part of Tovero.
 #
@@ -34,6 +34,9 @@
 # IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+# TODO: cleanup specifications, dimensions, accessors
+# TODO: BRL-CAD naming scheme
+
 #
 # Command line argument processing first
 #
@@ -49,14 +52,14 @@ else
 end
 
 # These load the extension libraries
-require 'libtovero_support_rb_0'
-require 'libtovero_math_rb_0'
-require 'libtovero_graphics_rb_0'
+require 'libtovero_support_rb_1'
+require 'libtovero_math_rb_1'
+require 'libtovero_graphics_rb_1'
 
 # These are like "using namespace" in C++
-include Libtovero_support_rb_0
-include Libtovero_math_rb_0
-include Libtovero_graphics_rb_0
+include Libtovero_support_rb_1
+include Libtovero_math_rb_1
+include Libtovero_graphics_rb_1
 
 # TODO: separate out as external Ruby module
 
@@ -102,18 +105,19 @@ rubber_color = Color.new(10, 10, 10, 0)
 $rubber_attributes = CombinationAttributes.new
 $rubber_attributes.shader = rubber_shader
 $rubber_attributes.color = rubber_color
+$rubber_attributes.is_part = true
 #   trim
-trim_shader = PhongShader::plastic
 trim_color = Color.new(255, 255, 255, 0)
 $trim_attributes = CombinationAttributes.new
-$trim_attributes.shader = trim_shader
 $trim_attributes.color = trim_color
+$trim_attributes.is_part = true
 #   glass
 glass_shader = PhongShader::glass
 glass_color = Color.new(200, 220, 220, 0)
 $glass_attributes = CombinationAttributes.new
 $glass_attributes.shader = glass_shader
 $glass_attributes.color = glass_color
+$glass_attributes.is_part = true
 #   body
 body_shader = PhongShader::plastic
 body_shader.reflected = Unitless.new(0.1)
@@ -121,18 +125,20 @@ body_color = Color.new(0, 160, 255, 0)
 $body_attributes = CombinationAttributes.new
 $body_attributes.shader = body_shader
 $body_attributes.color = body_color
-#   headlights
-headlight_shader = PhongShader::plastic
-headlight_color = Color.new(200, 200, 200, 0)
-$headlight_attributes = CombinationAttributes.new
-$headlight_attributes.shader = headlight_shader
-$headlight_attributes.color = headlight_color
+$body_attributes.is_part = true
+#   headlight glass
+headlight_glass_shader = PhongShader::glass
+headlight_glass_shader.transmitted = Unitless.new(0.1)
+headlight_glass_color = Color.new(130, 200, 210, 0)
+$headlight_glass_attributes = CombinationAttributes.new
+$headlight_glass_attributes.shader = headlight_glass_shader
+$headlight_glass_attributes.color = headlight_glass_color
+$headlight_glass_attributes.is_part = true
 #   wheel rim
-wheel_rim_shader = PhongShader::plastic
 wheel_rim_color = Color.new(180, 180, 180, 0)
 $wheel_rim_attributes = CombinationAttributes.new
-$wheel_rim_attributes.shader = wheel_rim_shader
 $wheel_rim_attributes.color = wheel_rim_color
+$wheel_rim_attributes.is_part = true
 #   light
 light_color = Color.new(255, 255, 255, 0)
 light_shader = LightShader.new
@@ -142,7 +148,7 @@ light_shader.beam_dispersion_angle = Angle::degree * 120.0
 $light_attributes = CombinationAttributes.new
 $light_attributes.shader = light_shader
 $light_attributes.color = light_color
-
+$light_attributes.is_part = true
 #   background color
 background_color = Color.new(224, 212, 190, 0)
 
@@ -176,6 +182,55 @@ class BusPart
   end
 end
 
+class Dish < BusPart
+  # instance variables
+  #   specifications
+  attr_accessor :curvature_radius
+  attr_accessor :disk_radius
+  attr_accessor :direction
+  attr_accessor :up
+  attr_accessor :offset
+  #
+  attr_reader :shape
+
+  # constructor
+  def initialize(curvature_radius,
+                 disk_radius,
+                 direction,
+                 up,
+                 offset,
+                 parent,
+                 name)
+    super(parent, name)
+    @curvature_radius = curvature_radius
+    @disk_radius = disk_radius
+    @direction = direction
+    @up = up
+    @offset = offset
+  end
+
+  def render
+    x = sqrt(@curvature_radius * @curvature_radius - @disk_radius * @disk_radius)
+    basis = @direction.cross(@up)
+    center_offset = @direction * x
+    sphere = Sphere.new(@offset - center_offset,
+                        @curvature_radius,
+                        "#{full_name}#{$separator}sphere.s")
+    direction_offset = @direction * @curvature_radius
+    up_offset = @up * @curvature_radius
+    basis_offset = basis * @curvature_radius
+    radius_offset = direction_offset + up_offset + basis_offset
+    box = Box.new(@offset - center_offset - radius_offset,
+                  direction_offset + center_offset,
+                  up_offset * Unitless.new(2.0),
+                  basis_offset * Unitless.new(2.0),
+                  "#{full_name}#{$separator}sphere.s")
+    @shape = SolidCombination.new("#{full_name}")
+    @shape << sphere
+    @shape << SolidMember.new(box, SolidOperator::Difference_op)
+  end
+end
+
 class BusBody < BusPart
   # instance variables
   #   specifications
@@ -203,17 +258,20 @@ class BusBody < BusPart
   end
 
   def calculate_dimensions
-    @radiusing_diameter = @radiusing_value * 2.0;
-    @radiused_length = @parent.length() - @radiusing_diameter
-    @top_radiused_length = @parent.top_length() - @radiusing_diameter
-    @radiused_width = @parent.width() - @radiusing_diameter
-    @radiused_height = @parent.height() - @radiusing_diameter
+    radiusing_diameter = @radiusing_value * 2.0
+    @radiused_length = @parent.length() - radiusing_diameter
+    @top_radiused_length = @parent.top_length() - radiusing_diameter
+    @radiused_width = @parent.width() - radiusing_diameter
+    @radiused_height = @parent.height() - radiusing_diameter
   end
 
   def render_frame
     radiusing_base = Point.new(@radiusing_value,
                                @radiusing_value,
                                @radiusing_value)
+    frame_radiusing_sphere = Sphere.new(radiusing_base,
+                                        @radiusing_value,
+                                        "#{full_name()}#{$separator}frame_radiusing_sphere.s")
     lower_frame_length_cylinder = Cylinder.new(radiusing_base,
                                                Vector.new(@radiused_length,
                                                           ZERO_M,
@@ -246,14 +304,10 @@ class BusBody < BusPart
                                                           ZERO_M),
                                                @radiusing_value,
                                                "#{full_name()}#{$separator}front_frame_height_cylinder.s")
-    frame_radiusing_sphere = Sphere.new(radiusing_base,
-                                        @radiusing_value,
-                                        "#{full_name()}#{$separator}frame_radiusing_sphere.s")
-    frame_width = SolidCombination.new("#{full_name()}#{$separator}frame_width.r")
+    frame_width = SolidCombination.new("#{full_name()}#{$separator}frame_width.c")
     frame_width << frame_width_cylinder
     frame_width << frame_radiusing_sphere
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(ZERO_M,
                               ZERO_M,
                               @radiused_width)
@@ -270,20 +324,18 @@ class BusBody < BusPart
                                        @front_rake_height_vector,
                                        @radiusing_value,
                                        "#{full_name()}#{$separator}front_rake_cylinder.s")
-    front_rake = SolidCombination.new("#{full_name()}#{$separator}frame_front_rake.r")
+    front_rake = SolidCombination.new("#{full_name()}#{$separator}frame_front_rake.c")
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(@front_rake_start_point.get(0) - @radiusing_value,
                               @front_rake_start_point.get(1) - @radiusing_value,
                               @front_rake_start_point.get(2) - @radiusing_value)
     front_rake << front_rake_cylinder
 
     # setup rear frame
-    @rear_frame = SolidCombination.new("#{full_name()}#{$separator}rear_frame.r")
+    @rear_frame = SolidCombination.new("#{full_name()}#{$separator}rear_frame.c")
     #   add rear frame width members
     @rear_frame << frame_width
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(ZERO_M,
                               @radiused_height,
                               ZERO_M)
@@ -291,7 +343,6 @@ class BusBody < BusPart
     #   add rear frame height cylinders
     @rear_frame << rear_frame_height_cylinder
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(ZERO_M,
                               ZERO_M,
                               @radiused_width)
@@ -299,55 +350,47 @@ class BusBody < BusPart
     #   add lower frame length cylinders
     @rear_frame << lower_frame_length_cylinder
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(ZERO_M,
                               ZERO_M,
                               @radiused_width)
     @rear_frame << SolidMember.new(SolidOperand.new(lower_frame_length_cylinder, transform))
     #   add upper frame length cylinders
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(ZERO_M,
                               @radiused_height,
                               ZERO_M)
     @rear_frame << SolidMember.new(SolidOperand.new(upper_frame_length_cylinder, transform))
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(ZERO_M,
                               @radiused_height,
                               @radiused_width)
     @rear_frame << SolidMember.new(SolidOperand.new(upper_frame_length_cylinder, transform))
 
     # setup front frame
-    @front_frame = SolidCombination.new("#{full_name()}#{$separator}front_frame.r")
+    @front_frame = SolidCombination.new("#{full_name()}#{$separator}front_frame.c")
     #   add front frame width members
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(@radiused_length,
                               ZERO_M,
                               ZERO_M)
     @front_frame << SolidMember.new(SolidOperand.new(frame_width, transform))
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(@radiused_length,
                               front_frame_height,
                               ZERO_M)
     @front_frame << SolidMember.new(SolidOperand.new(frame_width, transform))
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(@top_radiused_length,
                               @radiused_height,
                               ZERO_M)
     @front_frame << SolidMember.new(SolidOperand.new(frame_width, transform))
     #   add front frame height cylinders
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(@radiused_length,
                               ZERO_M,
                               ZERO_M)
     @front_frame << SolidMember.new(SolidOperand.new(front_frame_height_cylinder, transform))
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(@radiused_length,
                               ZERO_M,
                               @radiused_width)
@@ -355,14 +398,13 @@ class BusBody < BusPart
     #   add front rake pillars
     @front_frame << front_rake
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(ZERO_M,
                               ZERO_M,
                               @radiused_width)
     @front_frame << SolidMember.new(SolidOperand.new(front_rake, transform))
 
     #   add front and back frames to the main frame
-    @frame = SolidCombination.new("#{full_name()}#{$separator}frame.r")
+    @frame = SolidCombination.new("#{full_name()}#{$separator}frame.c")
     @frame << @front_frame
     @frame << @rear_frame
   end
@@ -449,21 +491,21 @@ class BusBody < BusPart
                                  @parent.width()),
                       "#{full_name()}#{$separator}wedge.s")
 
-    # setup the fill assemblies
+    # setup the fill combinations
     #   rear
-    @rear_fill = SolidCombination.new("#{full_name()}#{$separator}rear_fill.r")
+    @rear_fill = SolidCombination.new("#{full_name()}#{$separator}rear_fill.c")
     @rear_fill << rear_length_box
     @rear_fill << rear_width_box
     @rear_fill << rear_height_box
     #   front
-    @front_fill = SolidCombination.new("#{full_name()}#{$separator}front_fill.r")
+    @front_fill = SolidCombination.new("#{full_name()}#{$separator}front_fill.c")
     @front_fill << front_length_box
     @front_fill << front_width_box
     @front_fill << front_height_box
     @front_fill << front_rake_box
     @front_fill << wedge
     #   main fill
-    @fill = SolidCombination.new("#{full_name()}#{$separator}fill.r")
+    @fill = SolidCombination.new("#{full_name()}#{$separator}fill.c")
     #     add the front and rear
     @fill << @rear_fill
     @fill << @front_fill
@@ -479,23 +521,20 @@ class BusBody < BusPart
                                     wheel_well_radius,
                                     "#{full_name()}#{$separator}wheel_well_solid.s")
 
-    @wheel_wells_solid = SolidCombination.new("#{full_name()}#{$separator}wheel_wells_solid.r")
+    @wheel_wells_solid = SolidCombination.new("#{full_name()}#{$separator}wheel_wells_solid.c")
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(@parent.rear_wheel_x(),
                               ZERO_M,
                               ZERO_M)
     @wheel_wells_solid << SolidMember.new(SolidOperand.new(wheel_well_solid, transform))
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(@parent.rear_wheel_x() + @parent.wheel_base(),
                               ZERO_M,
                               ZERO_M)
     @wheel_wells_solid << SolidMember.new(SolidOperand.new(wheel_well_solid, transform))
 
-    wheel_well = SolidCombination.new("#{full_name()}#{$separator}wheel_well.s")
+    wheel_well = SolidCombination.new("#{full_name()}#{$separator}wheel_well.c")
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_scale((wheel_well_radius - @shell_thickness) / wheel_well_radius,
                         (wheel_well_radius - @shell_thickness) / wheel_well_radius,
                         Unitless.new(1.0))
@@ -506,15 +545,13 @@ class BusBody < BusPart
     wheel_well << SolidMember.new(SolidOperand.new(wheel_well_solid, transform), SolidOperator::Difference_op)
 
 
-    @wheel_wells = SolidCombination.new("#{full_name()}#{$separator}wheel_wells.r")
+    @wheel_wells = SolidCombination.new("#{full_name()}#{$separator}wheel_wells.c")
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(@parent.rear_wheel_x(),
                               ZERO_M,
                               ZERO_M)
     @wheel_wells << SolidMember.new(SolidOperand.new(wheel_well, transform))
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(@parent.rear_wheel_x() + @parent.wheel_base(),
                               ZERO_M,
                               ZERO_M)
@@ -523,7 +560,7 @@ class BusBody < BusPart
   end
 
   def render_window_boxes
-    @rear_window_boxes = SolidCombination.new("#{full_name()}#{$separator}rear_window_boxes.a")
+    @rear_window_boxes = SolidCombination.new("#{full_name()}#{$separator}rear_window_boxes.c")
 
     #     windshield box
     windshield_rear_x = @parent.top_length() - @radiusing_value
@@ -556,7 +593,7 @@ class BusBody < BusPart
                                                         @parent.width() + @radiusing_value),
                                               "#{full_name()}#{$separator}passenger_window_box.s")
     #     back left window box
-    back_window_rear_x = @radiusing_diameter
+    back_window_rear_x = @radiusing_value * 2.0
     back_left_window_box = AxisAlignedBox.new(Point.new(back_window_rear_x,
                                                         window_base_y,
                                                         -@radiusing_value),
@@ -608,23 +645,23 @@ class BusBody < BusPart
 
   def render_solid
     # body
-    @solid = Combination.new("#{full_name()}#{$separator}solid.r")
+    @solid = SolidCombination.new("#{full_name()}#{$separator}solid.c")
     @solid << @frame
     @solid << @fill
 
     # front
-    @front_solid = Combination.new("#{full_name()}#{$separator}front_solid.r")
+    @front_solid = SolidCombination.new("#{full_name()}#{$separator}front_solid.c")
     @front_solid << @front_frame
     @front_solid << @front_fill
 
     # rear
-    @rear_solid = Combination.new("#{full_name()}#{$separator}rear_solid.r")
+    @rear_solid = SolidCombination.new("#{full_name()}#{$separator}rear_solid.c")
     @rear_solid << @rear_frame
     @rear_solid << @rear_fill
   end
 
   def render
-    @assembly = Combination.new($body_attributes, "#{full_name()}.a")
+    combo = SolidCombination.new("#{full_name()}.c")
 
     render_frame
     render_fill
@@ -640,13 +677,16 @@ class BusBody < BusPart
                         @shell_thickness,
                         @shell_thickness)
 
-    @assembly << @solid
-    @assembly << SolidMember.new(SolidOperand.new(@solid, transform),
-                                 SolidOperator::Difference_op)
-    @assembly << SolidMember.new(@windshield_box, SolidOperator::Difference_op)
-    @assembly << SolidMember.new(@rear_window_boxes, SolidOperator::Difference_op)
-    @assembly << SolidMember.new(@wheel_wells_solid, SolidOperator::Difference_op)
-    @assembly << @wheel_wells
+    combo << @solid
+    combo << SolidMember.new(SolidOperand.new(@solid, transform),
+                             SolidOperator::Difference_op)
+    combo << SolidMember.new(@windshield_box, SolidOperator::Difference_op)
+    combo << SolidMember.new(@rear_window_boxes, SolidOperator::Difference_op)
+    combo << SolidMember.new(@wheel_wells_solid, SolidOperator::Difference_op)
+    combo << @wheel_wells
+
+    @assembly = Combination.new($body_attributes, "#{full_name()}.a")
+    @assembly << combo
   end
 end
 
@@ -688,7 +728,7 @@ class BusBumper < BusPart
   end
 
   def render
-    @part = SolidCombination.new("#{full_name()}#{$separator}bus_bumper.r")
+    @part = SolidCombination.new("#{full_name()}#{$separator}bus_bumper.c")
 
     cross_member_length = @parent.width() + @body_gap * 2.0 - @flattening_offset * 2.0
     cross_member = Cylinder.new(ORIGIN,
@@ -720,13 +760,11 @@ class BusBumper < BusPart
     @part << sphere
     @part << extension
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(ZERO_M,
                               ZERO_M,
                               cross_member_length)
     @part << SolidMember.new(SolidOperand.new(sphere, transform))
     transform = Transformation.new
-    transform.translation_units = M
     transform.set_translation(ZERO_M,
                               ZERO_M,
                               cross_member_length)
@@ -736,13 +774,13 @@ class BusBumper < BusPart
 end
 
 class BusHubCap < BusPart
-  # dimensions
-  attr_accessor :base_width
-  attr_accessor :base_height
-  # dependent geometry
-  attr_accessor :inner_cylinder
-  attr_accessor :outer_cylinder
-  #
+  # 
+  attr_accessor :curvature_radius
+  attr_accessor :disk_radius
+  attr_accessor :direction
+  attr_accessor :up
+  attr_accessor :base
+
   attr_reader :part
 
   # constructor
@@ -751,25 +789,16 @@ class BusHubCap < BusPart
   end
 
   def render
-    center_z = (@base_width / 2.0) * 1.2 - @base_height / 2.0
-    offset_z = -@base_width
-    radius = @base_height / 2.0
-
-    hub_cap_sphere = Sphere.new(Point.new(ZERO_M,
-                                          ZERO_M,
-                                          center_z),
-                                radius,
-                                "#{full_name()}#{$separator}hub_cap_sphere.s")
-    hub_cap_offset = Transformation.new
-    hub_cap_offset.translation_units = M
-    hub_cap_offset.set_translation(ZERO_M,
-                                   ZERO_M,
-                                   offset_z)
+    dish = Dish.new(@curvature_radius,
+                    @disk_radius,
+                    @direction,
+                    @up,
+                    @base,
+                    self,
+                    "dish")
+    dish.render()
     @part = Combination.new($trim_attributes, "#{full_name()}.r")
-    @part << hub_cap_sphere
-    @part << SolidMember.new(@inner_cylinder, SolidOperator::Intersection_op)
-    @part << SolidMember.new(SolidOperand.new(@outer_cylinder, hub_cap_offset),
-                             SolidOperator::Difference_op)
+    @part << dish.shape()
   end
 end
 
@@ -821,7 +850,7 @@ class BusTire < BusPart
   end
 
   def render
-    @part = Combination.new($rubber_attributes, "#{full_name()}.r")
+    combo = SolidCombination.new("#{full_name()}.c")
 
     torus_inner_radius = (@width * 1.1) / 2.0
     torus_outer_radius = (@height * 1.05) / 2.0 - torus_inner_radius
@@ -830,9 +859,12 @@ class BusTire < BusPart
                       torus_outer_radius,
                       torus_inner_radius,
                       "#{full_name()}#{$separator}torus.s")
-    @part << torus
-    @part << SolidMember.new(outer_center_cylinder, SolidOperator::Intersection_op)
-    @part << SolidMember.new(inner_center_cylinder, SolidOperator::Difference_op)
+    combo << torus
+    combo << SolidMember.new(outer_center_cylinder, SolidOperator::Intersection_op)
+    combo << SolidMember.new(inner_center_cylinder, SolidOperator::Difference_op)
+
+    @part = Combination.new($rubber_attributes, "#{full_name()}.r")
+    @part << combo
   end
 end
 
@@ -842,6 +874,8 @@ class BusWheelRim < BusPart
   attr_accessor :diameter
   attr_accessor :width
   #
+  attr_accessor :rim_offset
+  #
   attr_reader :part
 
   def initialize(parent, name)
@@ -852,12 +886,13 @@ class BusWheelRim < BusPart
     @part = Combination.new($wheel_rim_attributes, "#{full_name()}.r")
 
     wheel_rim_depth = @width / 1.1
+    @rim_offset = Vector.new(ZERO_M,
+                             ZERO_M,
+                             wheel_rim_depth)
     wheel_rim_cylinder = Cylinder.new(Point.new(ZERO_M,
                                                 ZERO_M,
                                                 -wheel_rim_depth / 2.0),
-                                      Vector.new(ZERO_M,
-                                                 ZERO_M,
-                                                 wheel_rim_depth),
+                                      @rim_offset,
                                       @diameter / 2.0,
                                       "#{full_name()}#{$separator}wheel_rim_cylinder.s")
     @part << wheel_rim_cylinder
@@ -866,7 +901,7 @@ end
 
 class BusWheel < BusPart
   # instance variables
-  #   dimensions
+  #   dependent dimensions
   attr_reader :height
   #
   attr_reader :tire
@@ -880,8 +915,6 @@ class BusWheel < BusPart
     @tire = BusTire.new(self, "tire")
     @wheel_rim = BusWheelRim.new(self, "wheel_rim")
     @hub_cap = BusHubCap.new(self, "hub_cap")
-
-    # assembly
   end
 
   def calculate_dimensions
@@ -891,18 +924,21 @@ class BusWheel < BusPart
     @wheel_rim.diameter = @tire.center_diameter()
     @wheel_rim.width = @tire.width()
     @height = @tire.height()
-    @hub_cap.base_height = @tire.height()
-    @hub_cap.base_width = @tire.width()
+    @hub_cap.disk_radius = @tire.center_diameter() / 2.0
+    @hub_cap.disk_radius = @hub_cap.disk_radius() - @hub_cap.disk_radius() / 10.0
+    @hub_cap.curvature_radius = @hub_cap.disk_radius() * 1.8
   end
 
   def render
     @tire.render
     @wheel_rim.render
-    # update the dependent geometry
-    @hub_cap.inner_cylinder = @tire.inner_center_cylinder()
-    @hub_cap.outer_cylinder = @tire.outer_center_cylinder()
+    # update dependent geometry
+    @hub_cap.base = ORIGIN + @wheel_rim.rim_offset() / Unitless.new(2.0)
+    direction = Z_AXIS
+    @wheel_rim.rim_offset().normalize(direction)
+    @hub_cap.direction = direction
+    @hub_cap.up = Y_AXIS
     @hub_cap.render
-
     @assembly = Combination.new("#{full_name()}.a")
     @assembly << @tire.part()
     @assembly << @wheel_rim.part()
@@ -913,7 +949,10 @@ end
 class BusHeadlights < BusPart
   # instance variables
   #   specifications
-  attr_accessor :headlight_sphere_radius
+  attr_accessor :curvature_radius
+  attr_accessor :disk_radius
+  attr_accessor :direction
+  attr_accessor :up
   #
   attr_reader :assembly
 
@@ -922,36 +961,76 @@ class BusHeadlights < BusPart
     super(parent, name)
 
     # default values for instance variables
-    @sphere_radius = M * 0.225
+    @curvature_radius = M * 0.300
+    @disk_radius = M * 0.15
+    @direction = X_AXIS
+    @up = Y_AXIS
   end
 
   def render
-    @assembly = Combination.new($headlight_attributes, "#{full_name()}.r")
-
-    sphere_offset_x = @parent.length() - M * 0.190
-    center_x = sphere_offset_x
-    center_y = @parent.headlight_height() - @parent.wheel().height() / 2.0
     offset_z = (@parent.width() - @parent.headlight_spacing) / 2.0
-    left_sphere = Sphere.new(Point.new(center_x,
-                                       center_y,
-                                       offset_z),
-                             @sphere_radius,
-                             "#{full_name()}#{$separator}left_sphere.s")
-    right_sphere = Sphere.new(Point.new(center_x,
-                                        center_y,
-                                        @parent.width() - offset_z),
-                              @sphere_radius,
-                              "#{full_name()}#{$separator}right_sphere.s")
-    box = AxisAlignedBox.new(Point.new(ZERO_M,
-                                       ZERO_M,
-                                       ZERO_M),
-                             Point.new(@parent.length() - @parent.body().shell_thickness() / 2.0,
-                                       @parent.height(),
-                                       @parent.width()),
-                             "#{full_name()}#{$separator}box.s")
-    @assembly << left_sphere
-    @assembly << right_sphere
-    @assembly << SolidMember.new(box, SolidOperator::Difference_op)
+    left_base = Point.new(@parent.length(),
+                          @parent.headlight_height() - @parent.wheel().height() / 2.0,
+                          offset_z)
+    left_dish = Dish.new(@curvature_radius,
+                         @disk_radius,
+                         @direction,
+                         @up,
+                         left_base,
+                         self,
+                         "left_dish.s")
+    left_dish.render()
+    glass_offset = Vector.new(M * 0.01,
+                              ZERO_M,
+                              ZERO_M)
+    left_glass = Dish.new(@curvature_radius * 0.6,
+                          @disk_radius - M * 0.03,
+                          @direction,
+                          @up,
+                          left_base + glass_offset,
+                          self,
+                         "left_glass.s")
+    left_glass.render()
+    left_trim = Combination.new($trim_attributes, "#{full_name()}#{$separator}left_trim.c")
+    left_trim << left_dish.shape()
+    left_trim << SolidMember.new(left_glass.shape(), SolidOperator::Difference_op)
+    left_glass_lens = Combination.new($headlight_glass_attributes, "#{full_name()}#{$separator}left_glass_lens.c")
+    left_glass_lens << left_glass.shape()
+    left_light = Combination.new("#{full_name()}#{$separator}left_light.c")
+    left_light << left_trim
+    left_light << left_glass_lens
+
+    right_base = Point.new(@parent.length(),
+                           @parent.headlight_height() - @parent.wheel().height() / 2.0,
+                           @parent.width() - offset_z)
+    right_dish = Dish.new(@curvature_radius,
+                           @disk_radius,
+                           @direction,
+                           @up,
+                           right_base,
+                           self,
+                          "right_dish.s")
+    right_dish.render()
+    right_glass = Dish.new(@curvature_radius * 0.6,
+                           @disk_radius - M * 0.03,
+                           @direction,
+                           @up,
+                           right_base + glass_offset,
+                           self,
+                           "right_glass.s")
+    right_glass.render()
+    right_trim = Combination.new($trim_attributes, "#{full_name()}#{$separator}right_trim.c")
+    right_trim << right_dish.shape()
+    right_trim << SolidMember.new(right_glass.shape(), SolidOperator::Difference_op)
+    right_glass_lens = Combination.new($headlight_glass_attributes, "#{full_name()}#{$separator}right_glass_lens.c")
+    right_glass_lens << right_glass.shape()
+    right_light = Combination.new("#{full_name()}#{$separator}right_light.c")
+    right_light << right_trim
+    right_light << right_glass_lens
+
+    @assembly = Combination.new("#{full_name()}#{$separator}headlights.c")
+    @assembly << left_light
+    @assembly << right_light
   end
 end
 
@@ -1017,7 +1096,7 @@ class Bus < BusPart
     @belt_line_height = M * 1.143
 
     # create a BRL-CAD database
-    @database = BCDatabase.new;
+    @database = BCDatabase.new
   end
 
   def render_body
@@ -1028,56 +1107,51 @@ class Bus < BusPart
 
   def render_wheels
     @wheel.render
-
     #     wheels
     front_wheel_x = @rear_wheel_x + @wheel_base
     left_wheel_z = (@width - @track - @wheel.tire().width()) / 2.0
     right_wheel_z = @width - left_wheel_z
     left_wheel_rotation_angle = Angle::degree * 180.0
     #       rear left
-    rear_left_wheel = SolidCombination.new("#{full_name()}#{$separator}rear_left_wheel.a")
+    rear_left_wheel = SolidCombination.new("#{full_name()}#{$separator}rear_left_wheel.c")
     wheel_transform = Transformation.new
-    wheel_transform.translation_units = M
     wheel_transform.set_rotation(Y_AXIS, left_wheel_rotation_angle)
     wheel_transform.translate(@rear_wheel_x,
                               ZERO_M,
                               left_wheel_z)
-    rear_left_wheel << SolidMember.new(SolidOperand.new(wheel.assembly(), wheel_transform))
+    rear_left_wheel << SolidMember.new(SolidOperand.new(@wheel.assembly(), wheel_transform))
 
     @assembly << rear_left_wheel
 
     #       front left
-    front_left_wheel = SolidCombination.new("#{full_name()}#{$separator}front_left_wheel.a")
+    front_left_wheel = SolidCombination.new("#{full_name()}#{$separator}front_left_wheel.c")
     wheel_transform = Transformation.new
-    wheel_transform.translation_units = M
     wheel_transform.set_rotation(Y_AXIS, left_wheel_rotation_angle - @wheel_turn_angle)
     wheel_transform.translate(front_wheel_x,
                               ZERO_M,
                               left_wheel_z)
-    front_left_wheel << SolidMember.new(SolidOperand.new(wheel.assembly(), wheel_transform))
+    front_left_wheel << SolidMember.new(SolidOperand.new(@wheel.assembly(), wheel_transform))
 
     @assembly << front_left_wheel
 
     #       rear right
-    rear_right_wheel = SolidCombination.new("#{full_name()}#{$separator}rear_right_wheel.a")
+    rear_right_wheel = SolidCombination.new("#{full_name()}#{$separator}rear_right_wheel.c")
     wheel_transform = Transformation.new
-    wheel_transform.translation_units = M
     wheel_transform.set_translation(@rear_wheel_x,
                                     ZERO_M,
                                     right_wheel_z)
-    rear_right_wheel << SolidMember.new(SolidOperand.new(wheel.assembly(), wheel_transform))
+    rear_right_wheel << SolidMember.new(SolidOperand.new(@wheel.assembly(), wheel_transform))
 
     @assembly << rear_right_wheel
 
     #       front right
-    front_right_wheel = SolidCombination.new("#{full_name()}#{$separator}front_right_wheel.a")
+    front_right_wheel = SolidCombination.new("#{full_name()}#{$separator}front_right_wheel.c")
     wheel_transform = Transformation.new
-    wheel_transform.translation_units = M
     wheel_transform.set_rotation(Y_AXIS, -@wheel_turn_angle)
     wheel_transform.translate(front_wheel_x,
                               ZERO_M,
                               right_wheel_z)
-    front_right_wheel << SolidMember.new(SolidOperand.new(wheel.assembly(), wheel_transform))
+    front_right_wheel << SolidMember.new(SolidOperand.new(@wheel.assembly(), wheel_transform))
 
     @assembly << front_right_wheel
   end
@@ -1085,7 +1159,7 @@ class Bus < BusPart
   def render_windows
     window = BusWindow.new(self, "window")
 
-    windshield_shell = Combination.new($glass_attributes, "#{full_name()}#{$separator}windshield_shell.a")
+    windshield_shell = SolidCombination.new("#{full_name()}#{$separator}windshield_shell.c")
     windshield_shell << @body.front_solid()
     transform = Transformation.new
     transform.set_scale((@length - window.glass_thickness() * 2.0) / @length,
@@ -1096,14 +1170,14 @@ class Bus < BusPart
                         window.glass_thickness())
     windshield_shell << SolidMember.new(SolidOperand.new(@body.front_solid(), transform),
                                         SolidOperator::Difference_op)
-    windshield = SolidCombination.new("#{full_name()}#{$separator}windshield.r")
+    windshield_shell << SolidMember.new(@body.windshield_box(), SolidOperator::Intersection_op)
+    windshield = Combination.new($glass_attributes, "#{full_name()}#{$separator}windshield.r")
     windshield << windshield_shell
-    windshield << SolidMember.new(@body.windshield_box(), SolidOperator::Intersection_op)
 
     @assembly << windshield
 
     #       the rest of the windows
-    rear_window_shell = Combination.new($glass_attributes, "#{full_name()}#{$separator}rear_window_shell.a")
+    rear_window_shell = SolidCombination.new("#{full_name()}#{$separator}rear_window_shell.c")
     rear_window_shell << @body.rear_solid()
     transform = Transformation.new
     transform.set_scale((@length - window.glass_thickness() * 2.0) / @length,
@@ -1114,10 +1188,10 @@ class Bus < BusPart
                         window.glass_thickness())
     rear_window_shell << SolidMember.new(SolidOperand.new(@body.rear_solid(), transform),
                                          SolidOperator::Difference_op)
+    rear_window_shell << SolidMember.new(@body.rear_window_boxes(), SolidOperator::Intersection_op)
 
-    rear_windows = SolidCombination.new("#{full_name()}#{$separator}rear_windows.r")
+    rear_windows = Combination.new($glass_attributes, "#{full_name()}#{$separator}rear_windows.r")
     rear_windows << rear_window_shell
-    rear_windows << SolidMember.new(@body.rear_window_boxes(), SolidOperator::Intersection_op)
 
     @assembly << rear_windows
   end
@@ -1127,7 +1201,6 @@ class Bus < BusPart
 
     rear_bumper = Combination.new($trim_attributes, "#{full_name()}#{$separator}rear_bumper.r")
     bumper_transform = Transformation.new
-    bumper_transform.translation_units = M
     bumper_transform.set_translation(- @bumper.body_gap() + @bumper.flattening_offset(),
                                      @bumper.radius() + @bumper.body_gap(),
                                      - @bumper.body_gap() + @bumper.flattening_offset())
@@ -1137,7 +1210,6 @@ class Bus < BusPart
 
     front_bumper = Combination.new($trim_attributes, "#{full_name()}#{$separator}front_bumper.r")
     bumper_transform = Transformation.new
-    bumper_transform.translation_units = M
     bumper_transform.set_rotation(Z_AXIS, Angle::degree * 180.0)
     bumper_transform.translate(@length + @bumper.body_gap() - @bumper.flattening_offset(),
                                @bumper.radius() + @bumper.body_gap() - @bumper.flattening_offset(),
@@ -1170,7 +1242,7 @@ class Bus < BusPart
     @rear_wheel_x = @rear_overhang + @wheel.height() / 2.0 + wheel_to_body_clearance
 
     # clear out any existing parts
-    solid_list = @database.top_solids;
+    solid_list = @database.top_solids
     solid_list.clear()
 
     # make a top level bounding box for the body as a reference
@@ -1198,11 +1270,10 @@ class Bus < BusPart
     # create a light
     light_shape = Sphere.new(ORIGIN,
                              M * 0.10,
-                             "#{$separator}light.s");
+                             "#{$separator}light.s")
     light = Combination.new($light_attributes, "#{$separator}light.r")
     #   adjust the direction of the light (points in the -Z direction by default) by rotating
     light_transform = Transformation.new
-    light_transform.translation_units = M
     light_transform.set_rotation(X_AXIS, Angle::degree * -90.0)
     light_transform.translate(Vector.new(@length / 2.0,
                                          @height + M * 4.0,
@@ -1223,7 +1294,9 @@ bus.render
 
 database_file = "bus.g"
 error = ErrorParam.new
-success = bus.database().write(database_file, true, error)
+success = bus.database().write(database_file,
+                               true,
+                               error)
 
 if (not success)
   puts("Could not write #{database_file}:")
@@ -1238,12 +1311,11 @@ else
     background = "#{background_color.red}/#{background_color.green}/#{background_color.blue}"
 
     # TODO: specify parameters in Ruby and generate string
-    # display raytraced model
+    # raytrace
     #   options to "rt":
     #     -A [ambient light fraction (0.0 - 1.0)]
     #     -J [jitter (1 = spatial)]
     #     -R [do not report overlaps]
-    #     -l [lighting model number (7 = photon mapping)]
     #     -C [background color (R/G/B)]
     #     -w [width in pixels of rendering]
     #     -n [height in pixels (number of lines) of rendering]
@@ -1253,20 +1325,21 @@ else
     #     (model/view info can be saved in mged with the "saveview" command)
 
     command = "rm -f #{database_file}.pix #{database_file}.png ; \
-              rt -A0.5 -J1 -R -l7 -C#{background} -o #{database_file}.pix \
+              rt -A0.5 -J1 -R -C#{background} -o #{database_file}.pix \
                 -w#{image_width} -n#{image_height} -M #{database_file} \
                 #{bus.assembly().name()} #{$separator}light.r <<EOF\n\
                 viewsize 9540;\n\
                 orientation -0.119 0.331 0.00625 0.936;\n\
                 eye_pt 12070 4500 12900;\n\
-                start 0; clean; end;\n\
-                EOF"
-    %x[#{command}]
+                start 0; clean; end;\n\EOF"
+    result = %x[#{command}]
 
-    # convert image and cleanup
-    command = "pix-png -w#{image_width} -n#{image_height} -o #{database_file}.png #{database_file}.pix ; \
+    if (result)
+      # convert image and cleanup
+      command = "pix-png -w#{image_width} -n#{image_height} -o #{database_file}.png #{database_file}.pix ; \
               rm -f #{database_file}.pix"
-    %x[#{command}]
+      %x[#{command}]
+    end
   else
     # load the interactive editor
     command = "mged #{database_file}"
