@@ -2,7 +2,7 @@
 #
 # whale_data.rb
 #
-# Copyright 2012-2013 Roan Trail, Inc.
+# Copyright 2012-2014 Roan Trail, Inc.
 #
 # This file is part of Tovero.
 #
@@ -34,7 +34,7 @@
 # IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-normals = VectorArray.new
+normals = UnitVectorArray.new
 face_normals = TriangleFaceArray.new
 
 m = Distance::meter
